<?php
/**
 * A Magento 2 module named FiloBlu/Cegid
 * Copyright (C) 2019  2019
 * 
 * This file is part of FiloBlu/Cegid.
 * 
 * FiloBlu/Cegid is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 */

namespace FiloBlu\Cegid\Model;

use FiloBlu\Cegid\Api\CegidRepositoryInterface;
use FiloBlu\Cegid\Api\Data\CegidSearchResultsInterfaceFactory;
use FiloBlu\Cegid\Api\Data\CegidInterfaceFactory;
use Magento\Framework\Api\DataObjectHelper;
use Magento\Framework\Exception\CouldNotDeleteException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Exception\CouldNotSaveException;
use Magento\Framework\Reflection\DataObjectProcessor;
use Magento\Framework\Api\SearchCriteria\CollectionProcessorInterface;
use FiloBlu\Cegid\Model\ResourceModel\Cegid as ResourceCegid;
use FiloBlu\Cegid\Model\ResourceModel\Cegid\CollectionFactory as CegidCollectionFactory;
use Magento\Store\Model\StoreManagerInterface;
use Magento\Framework\Api\ExtensionAttribute\JoinProcessorInterface;
use Magento\Framework\Api\ExtensibleDataObjectConverter;

class CegidRepository implements CegidRepositoryInterface
{

    protected $resource;

    protected $cegidFactory;

    protected $cegidCollectionFactory;

    protected $searchResultsFactory;

    protected $dataObjectHelper;

    protected $dataObjectProcessor;

    protected $dataCegidFactory;

    protected $extensionAttributesJoinProcessor;

    private $storeManager;

    private $collectionProcessor;

    protected $extensibleDataObjectConverter;

    /**
     * @param ResourceCegid $resource
     * @param CegidFactory $cegidFactory
     * @param CegidInterfaceFactory $dataCegidFactory
     * @param CegidCollectionFactory $cegidCollectionFactory
     * @param CegidSearchResultsInterfaceFactory $searchResultsFactory
     * @param DataObjectHelper $dataObjectHelper
     * @param DataObjectProcessor $dataObjectProcessor
     * @param StoreManagerInterface $storeManager
     * @param CollectionProcessorInterface $collectionProcessor
     * @param JoinProcessorInterface $extensionAttributesJoinProcessor
     * @param ExtensibleDataObjectConverter $extensibleDataObjectConverter
     */
    public function __construct(
        ResourceCegid $resource,
        CegidFactory $cegidFactory,
        CegidInterfaceFactory $dataCegidFactory,
        CegidCollectionFactory $cegidCollectionFactory,
        CegidSearchResultsInterfaceFactory $searchResultsFactory,
        DataObjectHelper $dataObjectHelper,
        DataObjectProcessor $dataObjectProcessor,
        StoreManagerInterface $storeManager,
        CollectionProcessorInterface $collectionProcessor,
        JoinProcessorInterface $extensionAttributesJoinProcessor,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ) {
        $this->resource = $resource;
        $this->cegidFactory = $cegidFactory;
        $this->cegidCollectionFactory = $cegidCollectionFactory;
        $this->searchResultsFactory = $searchResultsFactory;
        $this->dataObjectHelper = $dataObjectHelper;
        $this->dataCegidFactory = $dataCegidFactory;
        $this->dataObjectProcessor = $dataObjectProcessor;
        $this->storeManager = $storeManager;
        $this->collectionProcessor = $collectionProcessor;
        $this->extensionAttributesJoinProcessor = $extensionAttributesJoinProcessor;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    /**
     * {@inheritdoc}
     */
    public function save(
        \FiloBlu\Cegid\Api\Data\CegidInterface $cegid
    ) {
        /* if (empty($cegid->getStoreId())) {
            $storeId = $this->storeManager->getStore()->getId();
            $cegid->setStoreId($storeId);
        } */
        
        $cegidData = $this->extensibleDataObjectConverter->toNestedArray(
            $cegid,
            [],
            \FiloBlu\Cegid\Api\Data\CegidInterface::class
        );
        
        $cegidModel = $this->cegidFactory->create()->setData($cegidData);
        
        try {
            $this->resource->save($cegidModel);
        } catch (\Exception $exception) {
            throw new CouldNotSaveException(__(
                'Could not save the cegid: %1',
                $exception->getMessage()
            ));
        }
        return $cegidModel->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getById($cegidId)
    {
        $cegid = $this->cegidFactory->create();
        $this->resource->load($cegid, $cegidId);
        if (!$cegid->getId()) {
            throw new NoSuchEntityException(__('Cegid with id "%1" does not exist.', $cegidId));
        }
        return $cegid->getDataModel();
    }

    /**
     * {@inheritdoc}
     */
    public function getList(
        \Magento\Framework\Api\SearchCriteriaInterface $criteria
    ) {
        $collection = $this->cegidCollectionFactory->create();
        
        $this->extensionAttributesJoinProcessor->process(
            $collection,
            \FiloBlu\Cegid\Api\Data\CegidInterface::class
        );
        
        $this->collectionProcessor->process($criteria, $collection);
        
        $searchResults = $this->searchResultsFactory->create();
        $searchResults->setSearchCriteria($criteria);
        
        $items = [];
        foreach ($collection as $model) {
            $items[] = $model->getDataModel();
        }
        
        $searchResults->setItems($items);
        $searchResults->setTotalCount($collection->getSize());
        return $searchResults;
    }

    /**
     * {@inheritdoc}
     */
    public function delete(
        \FiloBlu\Cegid\Api\Data\CegidInterface $cegid
    ) {
        try {
            $cegidModel = $this->cegidFactory->create();
            $this->resource->load($cegidModel, $cegid->getCegidId());
            $this->resource->delete($cegidModel);
        } catch (\Exception $exception) {
            throw new CouldNotDeleteException(__(
                'Could not delete the Cegid: %1',
                $exception->getMessage()
            ));
        }
        return true;
    }

    /**
     * {@inheritdoc}
     */
    public function deleteById($cegidId)
    {
        return $this->delete($this->getById($cegidId));
    }
}
